@@ -14,14 +14,17 @@
 # limitations under the License.
 #
 import warnings
+from itertools import combinations
 from typing import Dict, Optional, Sequence, Union
 
 import tensorflow as tf
 from keras.layers.preprocessing import preprocessing_utils
+from keras.layers.preprocessing import preprocessing_utils as utils
+from keras.utils import layer_utils
 
 from merlin.models.config.schema import requires_schema
 from merlin.models.tf.core.base import Block, PredictionOutput
-from merlin.models.tf.core.combinators import TabularBlock
+from merlin.models.tf.core.combinators import ParallelBlock, TabularBlock
 from merlin.models.tf.typing import TabularData
 from merlin.models.tf.utils.tf_utils import (
     df_to_tensor,
@@ -32,15 +35,15 @@ from merlin.models.tf.utils.tf_utils import (
 from merlin.models.utils import schema_utils
 from merlin.schema import Schema, Tags
 
+ONE_HOT = utils.ONE_HOT
+MULTI_HOT = utils.MULTI_HOT
+COUNT = utils.COUNT
+
 
 @Block.registry.register("as-ragged")
 @tf.keras.utils.register_keras_serializable(package="merlin.models")
 class AsRaggedFeatures(TabularBlock):
-    """Convert all list-inputs to ragged-tensors.
-
-    By default, the dataloader will represent list-columns as a tuple of values & row-lengths.
-
-    """
+    """Convert all list (multi-hot/sequential) features to tf.RaggedTensor"""
 
     def call(self, inputs: TabularData, **kwargs) -> TabularData:
         outputs = {}
@@ -52,8 +55,19 @@ class AsRaggedFeatures(TabularBlock):
 
         return outputs
 
-    def compute_output_shape(self, input_shape):
-        return input_shape
+    def compute_output_shape(self, input_shapes):
+        output_shapes = {}
+        for k, v in input_shapes.items():
+            # If it is a list/sparse feature (in tuple representation), uses the offset as shape
+            if isinstance(v, tuple) and isinstance(v[1], tf.TensorShape):
+                output_shapes[k] = tf.TensorShape([v[1][0], None])
+            else:
+                output_shapes[k] = v
+
+        return output_shapes
+
+    def compute_call_output_shape(self, input_shapes):
+        return self.compute_output_shape(input_shapes)
 
 
 @Block.registry.register("as-sparse")
@@ -119,11 +133,10 @@ class AsDenseFeatures(TabularBlock):
         outputs = {}
 
         for key, val in input_shape.items():
-            if self.max_seq_length:
+            if isinstance(val, tuple):
                 outputs[key] = tf.TensorShape((batch_size, self.max_seq_length))
             else:
-                # TODO: What to do here?
-                raise ValueError("max_seq_length must be specified")
+                outputs[key] = tf.TensorShape((batch_size))
 
         return outputs
 
@@ -307,10 +320,7 @@ class RemovePad3D(Block):
                 predictions, tf.broadcast_to(tf.expand_dims(non_pad_mask, 1), tf.shape(predictions))
             )
 
-        return outputs.copy_with_updates(
-            predictions=predictions,
-            targets=targets,
-        )
+        return outputs.copy_with_updates(predictions=predictions, targets=targets)
 
 
 @tf.keras.utils.register_keras_serializable(package="merlin_models")
@@ -398,47 +408,202 @@ class ItemsPredictionWeightTying(Block):
         return logits
 
 
-@Block.registry.register_with_multiple_names("categorical_to_onehot")
+def reshape_categorical_input_tensor_for_encoding(
+    input, feat_name, features_2d_last_dim, output_mode, ensure_1d_for_one_hot_mode=True
+):
+    output = input
+    reshape_fn = tf.sparse.reshape if isinstance(output, tf.SparseTensor) else tf.reshape
+    if ensure_1d_for_one_hot_mode and output_mode == ONE_HOT:
+        if features_2d_last_dim.get(feat_name, None) == 1 or input.get_shape()[-1] == 1:
+            output = reshape_fn(output, [-1])
+        elif feat_name in features_2d_last_dim or (
+            input.get_shape()[-1] is not None and len(input.get_shape()) == 2
+        ):
+            raise ValueError(
+                "One-hot accepts input tensors that are squeezable to 1D, but received"
+                f" a tensor with shape: {input.get_shape()}"
+            )
+
+    else:
+        # if feat_name in features_2d_last_dim or len(input.get_shape()) == 2:
+        if feat_name in features_2d_last_dim or (
+            input.get_shape()[-1] is not None and len(input.get_shape()) == 2
+        ):
+            # Ensures that the shape is known to avoid error on graph mode
+            new_shape = (-1, features_2d_last_dim.get(feat_name, input.get_shape()[-1]))
+            output = reshape_fn(output, new_shape)
+        else:
+            expand_dims_fn = (
+                tf.sparse.expand_dims if isinstance(output, tf.SparseTensor) else tf.expand_dims
+            )
+            output = expand_dims_fn(output, 1)
+
+    return output
+
+
+@Block.registry.register_with_multiple_names("category_encoding")
 @tf.keras.utils.register_keras_serializable(package="merlin_models")
 @requires_schema
-class CategoricalOneHot(Block):
+class CategoryEncoding(TabularBlock):
     """
-    Transform categorical features (2-D and 3-D tensors) to a one-hot representation.
+    A preprocessing layer which encodes integer features.
+
+    This layer provides options for condensing data into a categorical encoding. It accepts integer
+    values as inputs, and it outputs a dense or sparse representation of those inputs. Only
+    categorical features with "CATEGORICAL" as Tag can be transformed, and other features without
+    this Tag would be discarded.
+    It outputs a TabularData (Dict of features), where each feature is a 2D tensor computed
+    based on the outputmode.
 
     Parameters:
     ----------
     schema : Optional[Schema]
         The `Schema` with the input features
+    output_mode: Optional[str]
+        Specification for the output of the layer. Defaults to `"multi_hot"`. Values can be
+        "one_hot", "multi_hot" or "count", configuring the transformation layer as follows:
+        - "one_hot": Encodes each individual element in the input into a tensor with shape
+            (batch_size, feature_cardinality), containing a 1 at the element index.
+            It accepts both 1D tensor or 2D tensor if squeezable (i.e., if the last dimension is 1).
+        - "multi_hot": Encodes each categorical value from the 2D input features into a
+            multi-hot representation with shape (batch_size, feature_cardinality), with 1 at
+            the indices present in the sequence and 0 for the other position.
+            If 1D feature is provided, it behaves the same as "one_hot".
+        - "count": also expects 2D tensor like `"multi_hot"` and outputs the features
+            with shape (batch_size, feature_cardinality). But instead of returning "multi-hot"
+            values, it outputs the frequency (count) of the number of items each item occurs
+            in each sample.
+    sparse: Optional[Boolean]
+        If true, returns a `SparseTensor` instead of a dense `Tensor`. Defaults to `False`.
+        Setting sparse=True is recommended for high-cardinality features, in order to avoid
+        out-of-memory errors.
+    count_weights: Optional[Union(tf.Tensor, tf.RaggedTensor, tf.SparseTensor)]
+        count_weights is used to calculate weighted sum of times a token at that index appeared when
+        `output_mode` is "count"
     """
 
-    def __init__(self, schema: Schema = None, **kwargs):
+    def __init__(
+        self,
+        schema: Schema = None,
+        output_mode="one_hot",
+        sparse=False,
+        count_weights=None,
+        **kwargs,
+    ):
         super().__init__(**kwargs)
         if schema:
-            self.set_schema(schema)
+            self.set_schema(schema.select_by_tag(Tags.CATEGORICAL))
+        self.sparse = sparse
         self.cardinalities = schema_utils.categorical_cardinalities(self.schema)
+        # 'output_mode' must be one of (COUNT, ONE_HOT, MULTI_HOT)
+        layer_utils.validate_string_arg(
+            output_mode,
+            allowable_strings=(COUNT, ONE_HOT, MULTI_HOT),
+            layer_name="CategoryEncoding",
+            arg_name="output_mode",
+        )
+        self.output_mode = output_mode
+        self.sparse = sparse
+        if count_weights is not None:
+            if self.output_mode != COUNT:
+                raise ValueError(
+                    "`count_weights` is not used when `output_mode` is not "
+                    "`'count'`. Received `count_weights={count_weights}`."
+                )
+            self.count_weights = utils.ensure_tensor(count_weights, self.compute_dtype)
+        else:
+            self.count_weights = None
 
-    def build(self, input_shapes):
-        super(CategoricalOneHot, self).build(input_shapes)
+        # Used to reshape 1D<->2Dtensors depending on the output_mode, when in graph mode
+        self.features_2d_last_dim = {}
 
     def call(self, inputs: TabularData, **kwargs) -> TabularData:
         outputs = {}
-        for name, val in self.cardinalities.items():
-            outputs[name] = tf.squeeze(tf.one_hot(inputs[name], val))
+        for name, depth in self.cardinalities.items():
+            # Ensures the input is a Tensor, SparseTensor, then convert to Tensor
+            if isinstance(inputs[name], tf.RaggedTensor):
+                raise ValueError(
+                    f"All `CategoryEncoding` inputs should not contain a RaggedTensor. Received "
+                    f"{name} with type of {type(inputs[name])}"
+                )
+
+            assertion_min_rank = tf.Assert(
+                tf.logical_and(
+                    tf.greater_equal(tf.rank(inputs[name]), 1),
+                    tf.less_equal(tf.rank(inputs[name]), 2),
+                ),
+                [
+                    "`CategoryEncoding` only accepts 1D or 2D-shaped inputs, but got "
+                    f"different rank for {name}"
+                ],
+            )
+
+            outputs[name] = utils.ensure_tensor(inputs[name])
+
+            if isinstance(outputs[name], tf.SparseTensor):
+                max_value = tf.reduce_max(outputs[name].values)
+                min_value = tf.reduce_min(outputs[name].values)
+            else:
+                max_value = tf.reduce_max(outputs[name])
+                min_value = tf.reduce_min(outputs[name])
+            condition = tf.logical_and(
+                tf.greater(tf.cast(depth, max_value.dtype), max_value),
+                tf.greater_equal(min_value, tf.cast(0, min_value.dtype)),
+            )
+            assertion_valid_values = tf.Assert(
+                condition,
+                [
+                    "Input values must be in the range 0 <= values < num_tokens"
+                    " with num_tokens={}".format(depth)
+                ],
+            )
+            with tf.control_dependencies([assertion_min_rank, assertion_valid_values]):
+                outputs[name] = reshape_categorical_input_tensor_for_encoding(
+                    outputs[name],
+                    name,
+                    self.features_2d_last_dim,
+                    self.output_mode,
+                    ensure_1d_for_one_hot_mode=True,
+                )
+
+                outputs[name] = utils.encode_categorical_inputs(
+                    outputs[name],
+                    output_mode=self.output_mode,
+                    depth=depth,
+                    dtype=self.compute_dtype,
+                    sparse=self.sparse,
+                    count_weights=self.count_weights,
+                )
+
+                if not self.sparse and isinstance(outputs[name], tf.SparseTensor):
+                    outputs[name] = tf.sparse.to_dense(outputs[name])
         return outputs
 
-    def compute_output_shape(self, input_shape):
+    def compute_output_shape(self, input_shapes):
         outputs = {}
-        for key, val in input_shape.items():
-            if len(val) == 3:
-                outputs[key] = tf.TensorShape((val[0], val[1], self.cardinalities[key]))
+        for key in self.schema.column_names:
+            input_shape = input_shapes[key]
+            if not input_shape:
+                outputs[key] = tf.TensorShape([self.cardinalities[key]])
             else:
-                outputs[key] = tf.TensorShape((val[0], self.cardinalities[key]))
+                outputs[key] = tf.TensorShape(input_shape[:1] + [self.cardinalities[key]])
+
+                if len(input_shape) == 2:
+                    self.features_2d_last_dim[key] = input_shape[-1]
+
         return outputs
 
     def get_config(self):
         config = super().get_config()
         if self.schema:
             config["schema"] = schema_utils.schema_to_tensorflow_metadata_json(self.schema)
+        config.update(
+            {
+                "output_mode": self.output_mode,
+                "sparse": self.sparse,
+                "count_weights": self.count_weights.numpy() if self.count_weights else None,
+            }
+        )
         return config
 
 
@@ -646,7 +811,7 @@ class PopularityLogitsCorrection(Block):
 @Block.registry.register("hashed_cross")
 @tf.keras.utils.register_keras_serializable(package="merlin.models")
 class HashedCross(TabularBlock):
-    """A transformation block which crosses categorical features using the "hasing trick".
+    """A transformation block which crosses categorical features using the "hashing trick".
     Conceptually, the transformation can be thought of as: hash(concatenation of features) %
     num_bins
     Example usage::
@@ -662,70 +827,103 @@ class HashedCross(TabularBlock):
         num_bins : int
             Number of hash bins.
         output_mode: string
-            Specification for the output of the layer. Defaults to
-            `"int"`.  Values can be `"int"`, or `"one_hot"` configuring the layer as
-            follows:
+            Specification for the output of the layer. Defaults to "one_hot".  Values can be "int",
+            or "one_hot", configuring the layer as follows:
             - `"int"`: Return the integer bin indices directly.
-            - `"one_hot"`: Encodes each individual element in the input into an
-                array the same size as `num_bins`, containing a 1 at the input's bin
-                index.
-        sparse : bool
-            Boolean. Only applicable to `"one_hot"` mode. If True, returns a
-            `SparseTensor` instead of a dense `Tensor`. Defaults to False.
-        output_name : string
+            - `"one_hot"`: Encodes each individual element in the input into an array with the same
+                size as `num_bins`, containing a 1 at the input's bin index.
+        sparse: bool
+            Boolean. Only applicable to `"one_hot"` mode. If True, returns a `SparseTensor` instead
+            of a dense `Tensor`. Defaults to False.
+        output_name: string
             Name of output feature, if not specified, default would be
             cross_<feature_name>_<feature_name>_<...>
+        infer_num_bins: bool
+            If True, num_bins would be set as the multiplier of feature cadinalities, if the
+            multiplier is bigger than max_num_bins, then it would be cliped by max_num_bins
+        max_num_bins: int
+            Upper bound of num_bins, by default 100000.
     """
 
     def __init__(
         self,
         schema: Schema,
-        num_bins: int,
+        num_bins: int = None,
         sparse: bool = False,
-        output_mode: str = "int",
+        output_mode: str = "one_hot",
         output_name: str = None,
+        infer_num_bins: bool = False,
+        max_num_bins: int = 100000,
         **kwargs,
     ):
         super().__init__(**kwargs)
 
-        if not (output_mode in ["int", "one_hot"]):
-            raise ValueError("output_mode must be 'int' or 'one_hot'")
+        if (not infer_num_bins) and (num_bins is None):
+            raise ValueError(
+                "num_bins is not given, and infer_num_bins is False, either of them "
+                "is required, if you want to set fixed num_bins, then set infer_num_bins to False,"
+                " and set num_bins to an integer value, if you want to infer num_bins from the "
+                "mulplier of feature cardinalities, at the same time you can set the max_num_bins."
+            )
+
+        if not (output_mode in ["int", "one_hot", "multi_hot"]):
+            raise ValueError("output_mode must be 'int', 'one_hot', or 'multi_hot'")
         self.schema = schema
-        self.num_bins = num_bins
         self.output_mode = output_mode
         self.sparse = sparse
         if not output_name:
             self.output_name = "cross"
-            for name in self.schema.column_names:
+            for name in sorted(schema.column_names):
                 self.output_name = self.output_name + "_" + name
         else:
             self.output_name = output_name
 
+        # Set num_bins
+        if num_bins:
+            self.num_bins = num_bins
+        else:
+            cardinalities = schema_utils.categorical_cardinalities(schema)
+            inferred_num_bins_from_cardinalities_multiplier = 1.0
+            for cardinality in cardinalities.values():
+                inferred_num_bins_from_cardinalities_multiplier = (
+                    inferred_num_bins_from_cardinalities_multiplier * cardinality
+                )
+            self.num_bins = int(min(max_num_bins, inferred_num_bins_from_cardinalities_multiplier))
+
+        # Used to enforce the shape of 2D tensors depending on the output_mode, when in graph mode
+        self.features_2d_last_dim = dict()
+
     def call(self, inputs):
         self._check_at_least_two_inputs()
+
         _inputs = {}
         for name in self.schema.column_names:
-            _inputs[name] = inputs[name]
-            rank = _inputs[name].shape.rank
-            if rank < 2:
-                _inputs[name] = tf.expand_dims(_inputs[name], -1)
-            if rank < 1:
-                _inputs[name] = tf.expand_dims(_inputs[name], -1)
+
+            assertion_min_rank = tf.Assert(
+                tf.logical_and(
+                    tf.greater_equal(tf.rank(inputs[name]), 1),
+                    tf.less_equal(tf.rank(inputs[name]), 2),
+                ),
+                [
+                    "`HashedCross` only accepts 1D or 2D-shaped inputs, but got "
+                    f"different rank for {name}"
+                ],
+            )
+
+            with tf.control_dependencies([assertion_min_rank]):
+                _inputs[name] = reshape_categorical_input_tensor_for_encoding(
+                    inputs[name],
+                    name,
+                    self.features_2d_last_dim,
+                    self.output_mode,
+                    ensure_1d_for_one_hot_mode=False,
+                )
 
         # Perform the cross and convert to dense
         output = tf.sparse.cross_hashed(list(_inputs.values()), self.num_bins)
-        output = tf.sparse.to_dense(output)
 
-        # Fix output shape and downrank to match input rank.
-        if rank == 2:
-            # tf.sparse.cross_hashed output shape will always be None on the last
-            # dimension. Given our input shape restrictions, we want to force shape 1
-            # instead.
-            output = tf.reshape(output, [-1, 1])
-        elif rank == 1:
-            output = tf.reshape(output, [-1])
-        elif rank == 0:
-            output = tf.reshape(output, [])
+        if self.output_mode == ONE_HOT:
+            output = tf.sparse.reshape(output, [-1])
 
         # Encode outputs.
         outputs = {}
@@ -735,15 +933,25 @@ class HashedCross(TabularBlock):
             depth=self.num_bins,
             sparse=self.sparse,
         )
+
+        if not self.sparse and isinstance(outputs[self.output_name], tf.SparseTensor):
+            outputs[self.output_name] = tf.sparse.to_dense(outputs[self.output_name])
         return outputs
 
     def compute_output_shape(self, input_shapes):
         self._check_at_least_two_inputs()
         self._check_input_shape_and_type(input_shapes)
+
+        # Save the last dim for 2D features so that we can reshape them in graph mode in call()
+        for key in self.schema.column_names:
+            input_shape = input_shapes[key]
+            if len(input_shape) == 2:
+                self.features_2d_last_dim[key] = input_shape[-1]
+
         output_shape = {}
-        one_input = list(input_shapes.values())[0]
+        batch_size = self.calculate_batch_size_from_input_shapes(input_shapes)
         output_shape[self.output_name] = preprocessing_utils.compute_shape_for_encode_categorical(
-            shape=one_input, output_mode=self.output_mode, depth=self.num_bins
+            shape=[batch_size, 1], output_mode=self.output_mode, depth=self.num_bins
         )
         return output_shape
 
@@ -764,7 +972,7 @@ class HashedCross(TabularBlock):
     def _check_at_least_two_inputs(self):
         if len(self.schema) < 2:
             raise ValueError(
-                "`HashedCrossing` should be called on at least two features. "
+                "`HashedCross` should be called on at least two features. "
                 f"Received: {len(self.schema)} schemas"
             )
         for name, column_schema in self.schema.column_schemas.items():
@@ -777,16 +985,129 @@ class HashedCross(TabularBlock):
     def _check_input_shape_and_type(self, inputs_shapes) -> TabularData:
         _inputs_shapes = []
         for name in self.schema.column_names:
-            _inputs_shapes.append(inputs_shapes[name])
-        first_shape = _inputs_shapes[0].as_list()
-        rank = len(first_shape)
-        if rank > 2 or (rank == 2 and first_shape[-1] != 1):
+            shape = inputs_shapes[name]
+
+            if shape.rank not in [1, 2]:
+                raise ValueError(
+                    "All `HashedCross` inputs should have 1D or 2D shape. "
+                    f"Received: input {name} with shape={shape}"
+                )
+
+            _inputs_shapes.append(shape)
+
+        if len(set([shape[0] for shape in _inputs_shapes])) > 1:
             raise ValueError(
-                "All `HashedCrossing` inputs should have shape `[]`, `[batch_size]` "
-                f"or `[batch_size, 1]`. Received: input {name} with shape={first_shape}"
-            )
-        if not all(x.as_list() == first_shape for x in _inputs_shapes):
-            raise ValueError(
-                "All `HashedCrossing` inputs should have equal shape. "
+                "All `HashedCross` inputs should have equal first dim (batch size). "
                 f"Received: inputs={_inputs_shapes}"
             )
+
+
+def HashedCrossAll(
+    schema: Schema,
+    num_bins: int = None,
+    infer_num_bins: bool = False,
+    max_num_bins: int = 100000,
+    max_level: int = 2,
+    sparse: bool = False,
+    output_mode: str = "one_hot",
+    ignore_combinations: Sequence[Sequence[str]] = [],
+) -> Block:
+    """Parallel block consists of HashedCross blocks for all combinations of schema with all levels
+        through level 2 to max_level.
+
+    Parameters:
+    ----------
+    schema: Schema
+        Schema of the input data.
+    max_level: int
+        Max num of levels, this function would hash cross all combinations, the number of features
+        included in these combinations is in the range from 2 to max_level, i.e. [2, max_level], by
+        default 2, which means it would return hashed cross blocks of all level 2 combinations of
+        features within schema
+
+        For example, if schemas contain 3 features: feature_1, feature_2 and feature_3, if we call
+            `level_3_cross = HashedCrossAll(schema = schemas, max_level = 3)`
+        Then level_3_cross is a Parallel block, which contains 4 hashed crosses of
+            1) feature_1 and feature_2
+            2) feature_1 and feature_3
+            3) feature_2 and feature_3
+            4) feature_1, feature_2 and feature_3
+    num_bins : int
+        Number of hash bins, note that num_bins is for all hashed cross transformation block, no
+        matter what level it is, if you want to set different num_bins for different hashed cross,
+        please use HashedCross to define each one with different num_bins.
+    output_mode: string
+        Specification for the output of the layer. Defaults to
+        `"one_hot"`.  Values can be `"int"`, or `"one_hot"` configuring the layer as
+        follows:
+        - `"int"`: Return the integer bin indices directly.
+        - `"one_hot"`: Encodes each individual element in the input into an
+            array the same size as `num_bins`, containing a 1 at the input's bin
+            index.
+    sparse : bool
+        Boolean. Only applicable to `"one_hot"` mode. If True, returns a
+        `SparseTensor` instead of a dense `Tensor`. Defaults to False.
+    infer_num_bins: bool
+        If True, all num_bins would be set as the multiplier of corresponding feature cadinalities,
+        if the multiplier is bigger than max_num_bins, then it would be cliped by max_num_bins
+    max_num_bins: int
+        Upper bound of num_bins for all hashed cross transformation blocks, by default 100000.
+
+    ignore_combinations :  Sequence[Sequence[str]]
+        If provided, ignore feature combinations from this list.
+        Useful to avoid interacting features whose combined value is always the same.
+        For example, interacting these features is not useful and one of the features
+        is dependent on the other :
+        [["item_id", "item_category"], ["user_id", "user_birth_city", "user_age"]]
+
+
+    Example usage::
+
+        level_3_cross = HashedCrossAll(schema = schemas, max_level = 3, infer_num_bins = True)
+    """
+
+    if max_level < 2 or max_level > 3:
+        raise ValueError(
+            "Please make sure 1 < max_level < 4, because the cross transformation requires at "
+            "least 2 features, and HashedCrossAll only support at most 3 level, if you want to "
+            "cross more than 3 features, please use HashedCross. Received: max_level = {max_level}"
+        )
+
+    if len(schema) < 2:
+        raise ValueError(
+            "`HashedCrossing` should be called on at least two features. "
+            f"Received: {len(schema)} schemas"
+        )
+
+    all_combinations = []
+    for combination_tuple in combinations(schema.column_names, 2):
+        all_combinations.append(set(combination_tuple))
+
+    if max_level == 3:
+        for combination_tuple in combinations(schema.column_names, 3):
+            all_combinations.append(set(combination_tuple))
+
+    if ignore_combinations:
+        ignore_combinations_set = list([set(c) for c in ignore_combinations])
+        all_combinations = list(
+            [
+                combination
+                for combination in all_combinations
+                if (combination not in ignore_combinations_set)
+            ]
+        )
+
+    hashed_crosses = []
+    for schema_names in all_combinations:
+        hashed_crosses.append(
+            HashedCross(
+                schema=schema.select_by_name(schema_names),
+                num_bins=num_bins,
+                sparse=sparse,
+                output_mode=output_mode,
+                infer_num_bins=infer_num_bins,
+                max_num_bins=max_num_bins,
+            )
+        )
+
+    return ParallelBlock(hashed_crosses)
