@@ -13,10 +13,10 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 #
-
+import inspect
 from copy import copy, deepcopy
 from dataclasses import dataclass
-from typing import Any, Callable, Dict, Optional, Union
+from typing import Any, Callable, Dict, Optional, Type, Union
 
 import tensorflow as tf
 from tensorflow.keras import backend
@@ -26,8 +26,9 @@ from tensorflow.python.tpu.tpu_embedding_v2_utils import FeatureConfig, TableCon
 import merlin.io
 from merlin.core.dispatch import DataFrameType
 from merlin.io import Dataset
+from merlin.models.tf.blocks.mlp import InitializerType, RegularizerType
 from merlin.models.tf.core.base import Block, BlockType
-from merlin.models.tf.core.combinators import SequentialBlock
+from merlin.models.tf.core.combinators import ParallelBlock, SequentialBlock
 from merlin.models.tf.core.tabular import (
     TABULAR_MODULE_PARAMS_DOCSTRING,
     Filter,
@@ -40,11 +41,12 @@ from merlin.models.tf.core.transformations import AsDenseFeatures, AsSparseFeatu
 # https://github.com/PyCQA/pylint/issues/3613
 # pylint: disable=no-value-for-parameter, unexpected-keyword-arg
 from merlin.models.tf.typing import TabularData
-from merlin.models.tf.utils.tf_utils import df_to_tensor
+from merlin.models.tf.utils.tf_utils import call_layer, df_to_tensor, list_col_to_ragged
 from merlin.models.utils import schema_utils
 from merlin.models.utils.doc_utils import docstring_parameter
 from merlin.models.utils.schema_utils import (
     create_categorical_column,
+    infer_embedding_dim,
     schema_to_tensorflow_metadata_json,
     tensorflow_metadata_json_to_schema,
 )
@@ -60,14 +62,20 @@ EMBEDDING_FEATURES_PARAMS_DOCSTRING = """
 
 
 class EmbeddingTableBase(Block):
-    def __init__(self, dim: int, col_schema: ColumnSchema, trainable=True, **kwargs):
+    def __init__(self, dim: int, *col_schemas: ColumnSchema, trainable=True, **kwargs):
         super(EmbeddingTableBase, self).__init__(trainable=trainable, **kwargs)
         self.dim = dim
+        self.features = {}
+        if len(col_schemas) == 0:
+            raise ValueError("At least one col_schema must be provided to the embedding table.")
 
-        if not col_schema.int_domain:
-            raise ValueError("`col_schema` needs to have a int-domain")
+        self.col_schema = col_schemas[0]
+        for col_schema in col_schemas:
+            self.add_feature(col_schema)
 
-        self.col_schema = col_schema
+    @property
+    def _schema(self):
+        return Schema([col_schema for col_schema in self.features.values()])
 
     @classmethod
     def from_pretrained(
@@ -83,21 +91,61 @@ class EmbeddingTableBase(Block):
     def input_dim(self):
         return self.col_schema.int_domain.max + 1
 
+    @property
+    def table_name(self):
+        return self.col_schema.int_domain.name or self.col_schema.name
+
+    def add_feature(self, col_schema: ColumnSchema) -> None:
+        """Add a feature to the table.
+
+        Adding more than one feature enables the table to lookup and return embeddings
+        for more than one feature when called with tabular data (Dict[str, TensorLike]).
+
+        Additional column schemas must have an int domain that matches the existing ones.
+
+        Parameters
+        ----------
+        col_schema : ColumnSchema
+        """
+        if not col_schema.int_domain:
+            raise ValueError("`col_schema` needs to have an int-domain")
+
+        if (
+            col_schema.int_domain.name
+            and self.col_schema.int_domain.name
+            and col_schema.int_domain.name != self.col_schema.int_domain.name
+        ):
+            raise ValueError(
+                "`col_schema` int-domain name does not match table domain name. "
+                f"{col_schema.int_domain.name} != {self.col_schema.int_domain.name} "
+            )
+
+        if col_schema.int_domain.max != self.col_schema.int_domain.max:
+            raise ValueError(
+                "`col_schema.int_domain.max` does not match existing input dim."
+                f"{col_schema.int_domain.max} != {self.col_schema.int_domain.max} "
+            )
+
+        self.features[col_schema.name] = col_schema
+
     def get_config(self):
         config = super().get_config()
         config["dim"] = self.dim
 
-        schema = schema_to_tensorflow_metadata_json(Schema([self.col_schema]))
+        schema = schema_to_tensorflow_metadata_json(self.schema)
         config["schema"] = schema
 
         return config
 
     @classmethod
     def from_config(cls, config):
+        dim = config.pop("dim")
         schema = tensorflow_metadata_json_to_schema(config.pop("schema"))
-        col_schema = schema.first
 
-        return cls(col_schema=col_schema, **config)
+        return cls(dim, *schema, **config)
+
+
+CombinerType = Union[str, tf.keras.layers.Layer]
 
 
 @tf.keras.utils.register_keras_serializable(package="merlin.models")
@@ -129,7 +177,7 @@ class EmbeddingTable(EmbeddingTableBase):
        `Flatten` then `Dense` layers upstream
        (without it, the shape of the dense outputs cannot be computed).
     combiner: A string specifying how to combine embedding results for each
-       entry. Currently "mean", "sqrtn" and "sum" are supported.
+       entry ("mean", "sqrtn" and "sum" are supported) or a layer.
        Default is None (no combiner used)
     trainable: Boolean, whether the layer's variables should be trainable.
     name: String name of the layer.
@@ -150,25 +198,30 @@ class EmbeddingTable(EmbeddingTableBase):
     def __init__(
         self,
         dim: int,
-        col_schema: ColumnSchema,
+        *col_schemas: ColumnSchema,
         embeddings_initializer="uniform",
         embeddings_regularizer=None,
         activity_regularizer=None,
         embeddings_constraint=None,
         mask_zero=False,
         input_length=None,
-        combiner: Optional[str] = None,
+        sequence_combiner: Optional[CombinerType] = None,
         trainable=True,
         name=None,
         dtype=None,
         dynamic=False,
         table=None,
-        weights=None,
         **kwargs,
     ):
         """Create an EmbeddingTable."""
         super(EmbeddingTable, self).__init__(
-            dim, col_schema, trainable=trainable, name=name, dtype=dtype, dynamic=dynamic, **kwargs
+            dim,
+            *col_schemas,
+            trainable=trainable,
+            name=name,
+            dtype=dtype,
+            dynamic=dynamic,
+            **kwargs,
         )
         if table is not None:
             self.table = table
@@ -179,16 +232,17 @@ class EmbeddingTable(EmbeddingTableBase):
                 activity_regularizer=activity_regularizer,
                 embeddings_constraint=embeddings_constraint,
                 mask_zero=mask_zero,
-                weights=weights,
                 input_length=input_length,
+                trainable=trainable,
             )
             self.table = tf.keras.layers.Embedding(
                 input_dim=self.input_dim,
                 output_dim=self.dim,
-                name=self.col_schema.name,
+                name=self.table_name,
                 **table_kwargs,
             )
-        self.combiner = combiner
+        self.sequence_combiner = sequence_combiner
+        self.supports_masking = True
 
     @classmethod
     def from_pretrained(
@@ -225,7 +279,7 @@ class EmbeddingTable(EmbeddingTableBase):
             dim,
             col_schema,
             name=name,
-            weights=[tf.Variable(embeddings, trainable=trainable)],
+            embeddings_initializer=tf.keras.initializers.constant(embeddings),
             trainable=trainable,
             **kwargs,
         )
@@ -235,62 +289,321 @@ class EmbeddingTable(EmbeddingTableBase):
         Invoked automatically before the first execution of `call()`.
         """
         self.table._maybe_build(inputs)
+
         return super(EmbeddingTable, self)._maybe_build(inputs)
 
-    def call(self, inputs, **kwargs):
+    def build(self, input_shapes):
+        if not self.table.built:
+            self.table.build(input_shapes)
+        return super(EmbeddingTable, self).build(input_shapes)
+
+    def call(
+        self, inputs: Union[tf.Tensor, TabularData], **kwargs
+    ) -> Union[tf.Tensor, TabularData]:
         """
         Parameters
         ----------
         inputs : Union[tf.Tensor, tf.RaggedTensor, tf.SparseTensor]
-            Tensors representing the input batch
+            Tensors or dictionary of tensors representing the input batch.
 
         Returns
         -------
-        A tensor corresponding to the embeddings for inputs
+        A tensor or dict of tensors corresponding to the embeddings for inputs
         """
-        dtype = backend.dtype(inputs)
-        if dtype != "int32" and dtype != "int64":
-            inputs = tf.cast(inputs, "int32")
-
-        if self.combiner:
-            if not isinstance(inputs, (tf.RaggedTensor, tf.SparseTensor)):
-                raise ValueError(
-                    "Combiner only supported for RaggedTensor and SparseTensor. "
-                    f"Received: {type(inputs)}"
-                )
-            if isinstance(inputs, tf.RaggedTensor):
-                inputs = inputs.to_sparse()
-            out = tf.nn.safe_embedding_lookup_sparse(
-                self.table.embeddings, inputs, None, combiner=self.combiner
-            )
-            if self._dtype_policy.compute_dtype != self._dtype_policy.variable_dtype:
-                # Instead of casting the variable as in most layers, cast the output, as
-                # this is mathematically equivalent but is faster.
-                out = tf.cast(out, self._dtype_policy.compute_dtype)
+        if isinstance(inputs, dict):
+            out = {}
+            for feature_name in self.schema.column_names:
+                if feature_name in inputs:
+                    out[feature_name] = self._call_table(inputs[feature_name], **kwargs)
         else:
-            if not isinstance(inputs, (tf.RaggedTensor, tf.Tensor)):
-                raise ValueError(
-                    "EmbeddingTable supports only RaggedTensor and Tensor input types. "
-                    f"Received: {type(inputs)}"
-                )
-            out = self.table(inputs)
+            out = self._call_table(inputs, **kwargs)
 
         return out
 
+    def _call_table(self, inputs, **kwargs):
+        if isinstance(inputs, tuple) and len(inputs) == 2:
+            inputs = list_col_to_ragged(inputs)
+
+        # Eliminating the last dim==1 of dense tensors before embedding lookup
+        if isinstance(inputs, tf.Tensor):
+            inputs = tf.squeeze(inputs, axis=-1)
+
+        if isinstance(inputs, (tf.RaggedTensor, tf.SparseTensor)):
+            if self.sequence_combiner and isinstance(self.sequence_combiner, str):
+                if isinstance(inputs, tf.RaggedTensor):
+                    inputs = inputs.to_sparse()
+                out = tf.nn.safe_embedding_lookup_sparse(
+                    self.table.embeddings, inputs, None, combiner=self.sequence_combiner
+                )
+            else:
+                if isinstance(inputs, tf.SparseTensor):
+                    raise ValueError(
+                        "Sparse tensors are not supported without sequence_combiner ",
+                        "please convert the tensor to a ragged or dense.",
+                    )
+
+                out = call_layer(self.table, inputs, **kwargs)
+                if isinstance(self.sequence_combiner, tf.keras.layers.Layer):
+                    out = call_layer(self.sequence_combiner, out, **kwargs)
+        else:
+            out = call_layer(self.table, inputs, **kwargs)
+
+        if self._dtype_policy.compute_dtype != self._dtype_policy.variable_dtype:
+            # Instead of casting the variable as in most layers, cast the output, as
+            # this is mathematically equivalent but is faster.
+            out = tf.cast(out, self._dtype_policy.compute_dtype)
+
+        return out
+
+    def compute_output_shape(
+        self, input_shape: Union[tf.TensorShape, Dict[str, tf.TensorShape]]
+    ) -> Union[tf.TensorShape, Dict[str, tf.TensorShape]]:
+        if isinstance(input_shape, dict):
+            output_shapes = {}
+            for feature_name in self.schema.column_names:
+                if feature_name in input_shape:
+                    output_shapes[feature_name] = self._compute_output_shape_table(
+                        input_shape[feature_name]
+                    )
+        else:
+            output_shapes = self._compute_output_shape_table(input_shape)
+
+        return output_shapes
+
+    def _compute_output_shape_table(
+        self, input_shape: Union[tf.TensorShape, tuple]
+    ) -> tf.TensorShape:
+        if isinstance(input_shape, tuple) and isinstance(input_shape[1], tf.TensorShape):
+            input_shape = tf.TensorShape([input_shape[1][0], None])
+
+        first_dims = input_shape
+        if (self.sequence_combiner is not None) or (input_shape.rank > 1 and input_shape[-1] == 1):
+            first_dims = input_shape[:-1]
+        output_shapes = tf.TensorShape(first_dims + [self.dim])
+
+        return output_shapes
+
+    def compute_call_output_shape(self, input_shapes):
+        return self.compute_output_shape(input_shapes)
+
     @classmethod
     def from_config(cls, config):
         config["table"] = tf.keras.layers.deserialize(config["table"])
+        if "combiner-layer" in config:
+            config["sequence_combiner"] = tf.keras.layers.deserialize(config.pop("combiner-layer"))
 
         return super().from_config(config)
 
     def get_config(self):
         config = super().get_config()
         config["table"] = tf.keras.layers.serialize(self.table)
-        config["combiner"] = self.combiner
+        if isinstance(self.sequence_combiner, tf.keras.layers.Layer):
+            config["combiner-layer"] = tf.keras.layers.serialize(self.sequence_combiner)
+        else:
+            config["sequence_combiner"] = self.sequence_combiner
 
         return config
 
 
+def Embeddings(
+    schema: Schema,
+    dim: Optional[Union[Dict[str, int], int]] = None,
+    infer_dim_fn: Callable[[ColumnSchema], int] = infer_embedding_dim,
+    sequence_combiner: Optional[Union[CombinerType, Dict[str, CombinerType]]] = "mean",
+    embeddings_initializer: Optional[Union[InitializerType, Dict[str, InitializerType]]] = None,
+    embeddings_regularizer: Optional[Union[RegularizerType, Dict[str, RegularizerType]]] = None,
+    activity_regularizer: Optional[Union[RegularizerType, Dict[str, RegularizerType]]] = None,
+    trainable: Optional[Union[bool, Dict[str, bool]]] = None,
+    table_cls: Type[tf.keras.layers.Layer] = EmbeddingTable,
+    pre: Optional[BlockType] = None,
+    post: Optional[BlockType] = None,
+    aggregation: Optional[TabularAggregationType] = None,
+    block_name: str = "embeddings",
+    **kwargs,
+) -> ParallelBlock:
+    """Creates a ParallelBlock with an EmbeddingTable for each categorical feature
+    in the schema.
+
+    Parameters
+    ----------
+    schema: Schema
+        Schema of the input data. This Schema object will be automatically generated using
+        [NVTabular](https://nvidia-merlin.github.io/NVTabular/main/Introduction.html).
+        Next to this, it's also possible to construct it manually.
+    dim: Optional[Union[Dict[str, int], int]], optional
+        A dim to use for all features, or a
+        Dict like {"feature_name": embedding size, ...}, by default None
+    infer_dim_fn: Callable[[ColumnSchema], int], defaults to infer_embedding_dim
+        The function to use to infer the embedding dimension, by default infer_embedding_dim
+    sequence_combiner: Optional[Union[str, tf.keras.layers.Layer]], optional
+       A string specifying how to combine embedding results for each
+       entry ("mean", "sqrtn" and "sum" are supported) or a layer.
+       Default is None (no combiner used)
+    embeddings_initializer: Union[InitializerType, Dict[str, InitializerType]], optional
+        An initializer function or a dict where keys are feature names and values are
+        callable to initialize embedding tables. Pre-trained embeddings can be fed via
+        embeddings_initializer arg.
+    embeddings_regularizer: Union[RegularizerType, Dict[str, RegularizerType]], optional
+        A regularizer function or a dict where keys are feature names and values are
+        callable to apply regularization to embedding tables.
+    activity_regularizer: Union[RegularizerType, Dict[str, RegularizerType]], optional
+        A regularizer function or a dict where keys are feature names and values are
+        callable to apply regularization to the activations of the embedding tables.
+    trainable: Optional[Dict[str, bool]] = None
+        Name of the column(s) whose embeddings should be frozen (or trainable) during training
+        trainable will be set to False/True for these column(s), accordingly
+    table_cls: Type[tf.keras.layers.Layer], by default EmbeddingTable
+        The class to use for each embedding table.
+    pre: Optional[BlockType], optional
+        Transformation block to apply before the embeddings lookup, by default None
+    post: Optional[BlockType], optional
+        Transformation block to apply after the embeddings lookup, by default None
+    aggregation: Optional[TabularAggregationType], optional
+        Transformation block to apply for aggregating the inputs, by default None
+    block_name: str, optional
+        Name of the block, by default "embeddings"
+
+    Returns
+    -------
+    ParallelBlock
+        Returns a parallel block with an embedding table for each categorical features
+    """
+    if trainable:
+        kwargs["trainable"] = trainable
+    if embeddings_initializer:
+        kwargs["embeddings_initializer"] = embeddings_initializer
+    if embeddings_regularizer:
+        kwargs["embeddings_regularizer"] = embeddings_regularizer
+    if activity_regularizer:
+        kwargs["activity_regularizer"] = activity_regularizer
+    if sequence_combiner:
+        kwargs["sequence_combiner"] = sequence_combiner
+
+    tables = {}
+
+    for col in schema:
+        table_kwargs = _forward_kwargs_to_table(col, table_cls, kwargs)
+        table_name = col.int_domain.name or col.name
+        if table_name in tables:
+            tables[table_name].add_feature(col)
+        else:
+            tables[table_name] = table_cls(
+                _get_dim(col, dim, infer_dim_fn), col, name=table_name, **table_kwargs
+            )
+
+    return ParallelBlock(
+        tables, pre=pre, post=post, aggregation=aggregation, name=block_name, schema=schema
+    )
+
+
+def _forward_kwargs_to_table(col, table_cls, kwargs):
+    arg_spec = inspect.getfullargspec(table_cls.__init__)
+    supported_kwargs = arg_spec.kwonlyargs
+    if arg_spec.defaults:
+        supported_kwargs += arg_spec.args[-len(arg_spec.defaults) :]
+
+    table_kwargs = {}
+    for key, val in kwargs.items():
+        if key in supported_kwargs:
+            if isinstance(val, dict):
+                if col.name in val:
+                    table_kwargs[key] = val[col.name]
+            else:
+                table_kwargs[key] = val
+
+    return table_kwargs
+
+
+def _get_dim(col, embedding_dims, infer_dim_fn):
+    dim = None
+    if isinstance(embedding_dims, dict):
+        dim = embedding_dims.get(col.name)
+    elif isinstance(embedding_dims, int):
+        dim = embedding_dims
+
+    if not dim:
+        dim = infer_dim_fn(col)
+
+    return dim
+
+
+class AverageEmbeddingsByWeightFeature(tf.keras.layers.Layer):
+    def __init__(self, weight_feature_name: str, axis=1, **kwargs):
+        """Computes the weighted average of a Tensor based
+        on one of the input features.
+        Typically used as a combiner for EmbeddingTable
+        for aggregating sequential embedding features
+
+        Parameters
+        ----------
+        weight_feature_name : str
+            Name of the feature to be used as weight for average
+        axis : int, optional
+            Axis for reduction, by default 1 (assuming the 2nd dim is
+            the sequence length)
+        """
+        super(AverageEmbeddingsByWeightFeature, self).__init__(**kwargs)
+        self.axis = axis
+        self.weight_feature_name = weight_feature_name
+
+    def call(self, inputs, features):
+        weight_feature = features[self.weight_feature_name]
+        if isinstance(inputs, tf.RaggedTensor) and not isinstance(weight_feature, tf.RaggedTensor):
+            raise ValueError(
+                f"If inputs is a tf.RaggedTensor, the weight feature ({self.weight_feature_name}) "
+                f"should also be a tf.RaggedTensor (and not a {type(weight_feature)}), "
+                "so that the list length can vary per example for both input embedding "
+                "and weight features."
+            )
+
+        weights = tf.expand_dims(tf.cast(weight_feature, tf.float32), -1)
+        output = tf.divide(
+            tf.reduce_sum(tf.multiply(inputs, weights), axis=self.axis),
+            tf.reduce_sum(weights, axis=self.axis),
+        )
+        return output
+
+    def compute_output_shape(self, input_shape):
+        return input_shape
+
+    @staticmethod
+    def from_schema_convention(schema: Schema, weight_features_name_suffix: str = "_weight"):
+        """Infers the weight features corresponding to sequential embedding
+        features based on the feature name suffix. For example, if a
+        sequential categorical feature is called `item_id_seq`, if there is another
+        feature in the schema called `item_id_seq_weight`, then it will be used
+        for weighted average. If a weight feature cannot be found for a given
+        seq cat. feature then standard mean is used as combiner
+
+
+        Parameters
+        ----------
+        schema : Schema
+            The feature schema
+        weight_features_name_suffix : str
+            Suffix to look for a corresponding weight feature
+
+        Returns
+        -------
+        Dict[str, WeightedAverageByFeature]
+            A dict where the key is the sequential categorical feature name and the value
+            is an instance of WeightedAverageByFeature with the corresponding weight feature name
+        """
+        cat_cols = schema.select_by_tag(Tags.CATEGORICAL)
+        seq_combiners = {}
+        for cat_col in cat_cols:
+            combiner = None
+            if Tags.SEQUENCE in cat_col.tags:
+                weight_col_name = f"{cat_col.name}{weight_features_name_suffix}"
+                if weight_col_name in schema.column_names:
+                    combiner = AverageEmbeddingsByWeightFeature(weight_col_name)
+                else:
+                    combiner = tf.keras.layers.Lambda(lambda x: tf.reduce_mean(x, axis=1))
+                seq_combiners[cat_col.name] = combiner
+
+        return seq_combiners
+
+
 @dataclass
 class EmbeddingOptions:
     embedding_dims: Optional[Dict[str, int]] = None
@@ -299,7 +612,7 @@ class EmbeddingOptions:
     infer_embedding_sizes_multiplier: float = 2.0
     infer_embeddings_ensure_dim_multiple_of_8: bool = False
     embeddings_initializers: Optional[
-        Union[Dict[str, Callable[[Any], None]], Callable[[Any], None]]
+        Union[Dict[str, Callable[[Any], None]], Callable[[Any], None], str]
     ] = None
     embeddings_l2_reg: float = 0.0
     combiner: Optional[str] = "mean"
