@@ -1,9 +1,11 @@
 from __future__ import annotations
 
+import collections
 import inspect
 import sys
+import warnings
 from collections.abc import Sequence as SequenceCollection
-from typing import TYPE_CHECKING, Dict, List, Optional, Protocol, Union, runtime_checkable
+from typing import TYPE_CHECKING, Dict, List, Optional, Protocol, Sequence, Union, runtime_checkable
 
 import six
 import tensorflow as tf
@@ -16,15 +18,21 @@ from merlin.models.tf.core.base import Block, ModelContext, PredictionOutput, is
 from merlin.models.tf.core.combinators import SequentialBlock
 from merlin.models.tf.core.prediction import Prediction, PredictionContext
 from merlin.models.tf.core.tabular import TabularBlock
-from merlin.models.tf.core.transformations import AsDenseFeatures, AsRaggedFeatures
+from merlin.models.tf.core.transformations import AsRaggedFeatures
 from merlin.models.tf.dataset import BatchedDataset
 from merlin.models.tf.inputs.base import InputBlock
 from merlin.models.tf.losses.base import loss_registry
-from merlin.models.tf.metrics.topk import filter_topk_metrics
+from merlin.models.tf.metrics.topk import TopKMetricsAggregator, filter_topk_metrics, split_metrics
 from merlin.models.tf.models.utils import parse_prediction_tasks
 from merlin.models.tf.prediction_tasks.base import ParallelPredictionBlock, PredictionTask
+from merlin.models.tf.predictions.base import ContrastivePredictionBlock, PredictionBlock
+from merlin.models.tf.typing import TabularData
 from merlin.models.tf.utils.search_utils import find_all_instances_in_layers
-from merlin.models.tf.utils.tf_utils import call_layer, maybe_deserialize_keras_objects
+from merlin.models.tf.utils.tf_utils import (
+    call_layer,
+    get_sub_blocks,
+    maybe_serialize_keras_objects,
+)
 from merlin.models.utils.dataset import unique_rows_by_features
 from merlin.schema import Schema, Tags
 
@@ -67,6 +75,9 @@ class ModelBlock(Block, tf.keras.Model):
     def __init__(self, block: Block, **kwargs):
         super().__init__(**kwargs)
         self.block = block
+        if hasattr(self, "set_schema"):
+            block_schema = getattr(block, "schema", None)
+            self.set_schema(block_schema)
 
     def call(self, inputs, **kwargs):
         if "features" not in kwargs:
@@ -288,45 +299,130 @@ class BaseModel(tf.keras.Model):
             initial_value=lambda: False,
         )
 
-        self.output_names = [task.task_name for task in self.prediction_tasks]
+        num_v1_blocks = len(self.prediction_tasks)
+        num_v2_blocks = len(self.prediction_blocks)
 
-        _metrics = {}
-        if isinstance(metrics, (list, tuple)) and len(self.prediction_tasks) == 1:
-            _metrics = {task.task_name: metrics for task in self.prediction_tasks}
+        if num_v1_blocks > 1 and num_v2_blocks > 1:
+            raise ValueError(
+                "You cannot use both `prediction_tasks` and `prediction_blocks` at the same time.",
+                "`prediction_tasks` is deprecated and will be removed in a future version.",
+            )
 
-        # If metrics are not provided, use the defaults from the prediction-tasks.
-        # TODO: Do the same for weight_metrics.
-        if not metrics:
-            for task_name, task in self.prediction_tasks_by_name().items():
-                _metrics[task_name] = [
-                    m() if inspect.isclass(m) else m for m in task.DEFAULT_METRICS
-                ]
+        if num_v1_blocks > 0:
+            self.output_names = [task.task_name for task in self.prediction_tasks]
+        else:
+            self.output_names = [block.full_name for block in self.prediction_blocks]
+            negative_sampling = kwargs.pop("negative_sampling", None)
+            if negative_sampling:
+                if not isinstance(self.prediction_blocks[0], ContrastivePredictionBlock):
+                    raise ValueError(
+                        "Negative sampling strategy can be used only with a"
+                        " `ContrastivePredictionBlock` prediction block"
+                    )
+                self.prediction_blocks[0].compile(negative_sampling=negative_sampling)
 
-        _loss = {}
-        if isinstance(loss, (tf.keras.losses.Loss, str)) and len(self.prediction_tasks) == 1:
-            _loss = {task.task_name: loss for task in self.prediction_tasks}
-
-        # If loss is not provided, use the defaults from the prediction-tasks.
-        if not loss:
-            for task_name, task in self.prediction_tasks_by_name().items():
-                _loss[task_name] = task.DEFAULT_LOSS
-
-        for key in _loss:
-            if isinstance(_loss[key], str) and _loss[key] in loss_registry:
-                _loss[key] = loss_registry.parse(_loss[key])
+        # This flag will make Keras change the metric-names which is not needed in v2
+        from_serialized = kwargs.pop("from_serialized", num_v2_blocks > 0)
 
         super(BaseModel, self).compile(
             optimizer=optimizer,
-            loss=_loss,
-            metrics=_metrics,
-            weighted_metrics=weighted_metrics,
+            loss=self._create_loss(loss),
+            metrics=self._create_metrics(metrics),
+            weighted_metrics=self._create_weighted_metrics(weighted_metrics),
             run_eagerly=run_eagerly,
             loss_weights=loss_weights,
             steps_per_execution=steps_per_execution,
             jit_compile=jit_compile,
+            from_serialized=from_serialized,
             **kwargs,
         )
 
+    def _create_metrics(self, metrics=None):
+        out = {}
+
+        num_v1_blocks = len(self.prediction_tasks)
+        if isinstance(metrics, dict):
+            out = metrics
+
+        elif isinstance(metrics, (list, tuple)):
+            # Retrieve top-k metrics & wrap them in TopKMetricsAggregator
+            topk_metrics, topk_aggregators, other_metrics = split_metrics(metrics)
+            if len(topk_metrics) > 0:
+                topk_aggregators.append(TopKMetricsAggregator(*topk_metrics))
+            metrics = other_metrics + topk_aggregators
+
+            if num_v1_blocks > 0:
+                if num_v1_blocks == 1:
+                    out[self.prediction_tasks[0].task_name] = metrics
+                else:
+                    for i, task in enumerate(self.prediction_tasks):
+                        out[task.task_name] = metrics[i]
+            else:
+                if len(self.prediction_blocks) == 1:
+                    out[self.prediction_blocks[0].full_name] = metrics
+                else:
+                    for i, block in enumerate(self.prediction_blocks):
+                        out[block.full_name] = metrics[i]
+
+        elif metrics is None:
+            for task_name, task in self.prediction_tasks_by_name().items():
+                out[task_name] = [m() if inspect.isclass(m) else m for m in task.DEFAULT_METRICS]
+
+            for prediction_name, prediction_block in self.predictions_by_name().items():
+                out[prediction_name] = prediction_block.default_metrics_fn()
+                if len(self.prediction_blocks) > 1:
+                    for metric in out[prediction_name]:
+                        metric._name = "/".join([prediction_block.full_name, metric.name])
+
+        return out
+
+    def _create_weighted_metrics(self, weighted_metrics=None):
+        out = {}
+
+        num_v1_blocks = len(self.prediction_tasks)
+
+        if isinstance(weighted_metrics, dict):
+            out = weighted_metrics
+
+        elif isinstance(weighted_metrics, (list, tuple)):
+            if num_v1_blocks > 0:
+                if num_v1_blocks == 1:
+                    out[self.prediction_tasks[0].task_name] = weighted_metrics
+                else:
+                    for i, task in enumerate(self.prediction_tasks):
+                        out[task.task_name] = weighted_metrics[i]
+            else:
+                if len(self.prediction_blocks) == 1:
+                    out[self.prediction_blocks[0].full_name] = weighted_metrics
+                else:
+                    for i, block in enumerate(self.prediction_blocks):
+                        out[block.full_name] = weighted_metrics[i]
+
+        return out
+
+    def _create_loss(self, loss=None):
+        out = {}
+
+        if isinstance(loss, (tf.keras.losses.Loss, str)):
+            if len(self.prediction_tasks) == 1:
+                out = {task.task_name: loss for task in self.prediction_tasks}
+            elif len(self.prediction_blocks) == 1:
+                out = {task.name: loss for task in self.prediction_blocks}
+
+        # If loss is not provided, use the defaults from the prediction-tasks.
+        if not loss:
+            for task_name, task in self.prediction_tasks_by_name().items():
+                out[task_name] = task.DEFAULT_LOSS
+
+            for task_name, task in self.predictions_by_name().items():
+                out[task_name] = task.default_loss
+
+        for key in out:
+            if isinstance(out[key], str) and out[key] in loss_registry:
+                out[key] = loss_registry.parse(out[key])
+
+        return out
+
     @property
     def prediction_tasks(self) -> List[PredictionTask]:
         from merlin.models.tf.prediction_tasks.base import PredictionTask
@@ -339,8 +435,43 @@ class BaseModel(tf.keras.Model):
         return {task.task_name: task for task in self.prediction_tasks}
 
     def prediction_tasks_by_target(self) -> Dict[str, List[PredictionTask]]:
+        """Method to index the model's prediction tasks by target names.
+
+        Returns
+        -------
+        Dict[str, List[PredictionTask]]
+            List of prediction tasks.
+        """
         outputs: Dict[str, Union[PredictionTask, List[PredictionTask]]] = {}
         for task in self.prediction_tasks:
+            if task.target_name in outputs:
+                if isinstance(outputs[task.target_name], list):
+                    outputs[task.target].append(task)
+                else:
+                    outputs[task.target_name] = [outputs[task.target_name], task]
+            outputs[task.target] = task
+
+        return outputs
+
+    @property
+    def prediction_blocks(self) -> List[PredictionBlock]:
+        results = find_all_instances_in_layers(self, PredictionBlock)
+
+        return results
+
+    def predictions_by_name(self) -> Dict[str, PredictionBlock]:
+        return {task.full_name: task for task in self.prediction_blocks}
+
+    def predictions_by_target(self) -> Dict[str, List[PredictionBlock]]:
+        """Method to index the model's prediction blocks by target names.
+
+        Returns
+        -------
+        Dict[str, List[PredictionBlock]]
+            List of prediction blocks.
+        """
+        outputs: Dict[str, List[PredictionBlock]] = {}
+        for task in self.prediction_blocks:
             if task.target in outputs:
                 if isinstance(outputs[task.target], list):
                     outputs[task.target].append(task)
@@ -351,8 +482,35 @@ class BaseModel(tf.keras.Model):
         return outputs
 
     def call_train_test(
-        self, x, y=None, training=False, testing=False, **kwargs
-    ) -> PredictionOutput:
+        self,
+        x: TabularData,
+        y: Optional[Union[tf.tensor, TabularData]] = None,
+        sample_weight=Optional[Union[float, tf.Tensor]],
+        training: bool = False,
+        testing: bool = False,
+        **kwargs,
+    ) -> Union[Prediction, PredictionOutput]:
+        """Apply the model's call method during Train or Test modes and prepare
+        Prediction (v2) or PredictionOutput (v1 - depreciated) objects
+
+        Parameters
+        ----------
+        x : TabularData
+            Dictionary of raw input features.
+        y : Union[tf.tensor, TabularData], optional
+            Target tensors, by default None
+        training : bool, optional
+            Flag for train mode, by default False
+        sample_weight : Union[float, tf.Tensor], optional
+            Sample weights to be used by the loss and by weighted_metrics
+        testing : bool, optional
+            Flag for test mode, by default False
+
+        Returns
+        -------
+        Union[Prediction, PredictionOutput]
+        """
+
         forward = self(
             x,
             targets=y,
@@ -360,47 +518,78 @@ class BaseModel(tf.keras.Model):
             testing=testing,
             **kwargs,
         )
-        if not self.prediction_tasks:
+        if not (self.prediction_tasks or self.prediction_blocks):
             return PredictionOutput(forward, y)
 
-        predictions, targets, output = {}, {}, None
-        for task in self.prediction_tasks:
-            task_x = forward
-            if isinstance(forward, dict) and task.task_name in forward:
-                task_x = forward[task.task_name]
-            if isinstance(task_x, PredictionOutput):
-                output = task_x
-                task_y = task_x.targets
-                task_x = task_x.predictions
+        predictions, targets, sample_weights, output = {}, {}, {}, None
+        # V1
+        if self.prediction_tasks:
+            for task in self.prediction_tasks:
+                task_x = forward
+                if isinstance(forward, dict) and task.task_name in forward:
+                    task_x = forward[task.task_name]
+
+                if isinstance(task_x, PredictionOutput):
+                    output = task_x
+                    task_y = output.targets
+                    task_x = output.predictions
+                    task_sample_weight = (
+                        sample_weight if output.sample_weight is None else output.sample_weight
+                    )
+                else:
+                    task_y = y[task.target_name] if isinstance(y, dict) and y else y
+                    task_sample_weight = sample_weight
+
+                targets[task.task_name] = task_y
+                predictions[task.task_name] = task_x
+                sample_weights[task.task_name] = task_sample_weight
+
+            if len(predictions) == 1 and len(targets) == 1:
+                predictions = list(predictions.values())[0]
+                targets = list(targets.values())[0]
+                sample_weights = list(sample_weights.values())[0]
+
+            if output:
+                return output.copy_with_updates(predictions, targets, sample_weight=sample_weights)
             else:
-                task_y = y[task.target_name] if isinstance(y, dict) and y else y
+                return PredictionOutput(predictions, targets, sample_weight=sample_weights)
 
-            targets[task.task_name] = task_y
-            predictions[task.task_name] = task_x
+        # V2
+        for task in self.prediction_blocks:
+            task_x = forward
+            if isinstance(forward, dict) and task.full_name in forward:
+                task_x = forward[task.full_name]
+            if isinstance(task_x, Prediction):
+                output = task_x
+                task_y = output.targets
+                task_x = output.outputs
+                task_sample_weight = (
+                    sample_weight if output.sample_weight is None else output.sample_weight
+                )
+            else:
+                task_y = y[task.target] if isinstance(y, dict) and y else y
+                task_sample_weight = sample_weight
 
-        if len(predictions) == 1 and len(targets) == 1:
-            predictions = predictions[list(predictions.keys())[0]]
-            targets = targets[list(targets.keys())[0]]
+            targets[task.full_name] = task_y
+            predictions[task.full_name] = task_x
+            sample_weights[task.full_name] = task_sample_weight
 
-        if output:
-            return output.copy_with_updates(predictions, targets)
-
-        return PredictionOutput(predictions, targets)
+        return Prediction(predictions, targets, sample_weights)
 
     def train_step(self, data):
         """Custom train step using the `compute_loss` method."""
 
         with tf.GradientTape() as tape:
             x, y, sample_weight = unpack_x_y_sample_weight(data)
-            outputs = self.call_train_test(x, y, training=True)
-            loss = self.compute_loss(x, outputs.targets, outputs.predictions, sample_weight)
+            outputs = self.call_train_test(x, y, sample_weight=sample_weight, training=True)
+            loss = self.compute_loss(x, outputs.targets, outputs.predictions, outputs.sample_weight)
 
         self._validate_target_and_loss(outputs.targets, loss)
 
         # Run backwards pass.
         self.optimizer.minimize(loss, self.trainable_variables, tape=tape)
 
-        metrics = self.compute_metrics(outputs, sample_weight, training=True)
+        metrics = self.compute_metrics(outputs, training=True)
         # Adding regularization loss to metrics
         metrics["regularization_loss"] = tf.reduce_sum(cast_losses_to_common_dtype(self.losses))
 
@@ -410,15 +599,15 @@ class BaseModel(tf.keras.Model):
         """Custom test step using the `compute_loss` method."""
 
         x, y, sample_weight = unpack_x_y_sample_weight(data)
-        outputs = self.call_train_test(x, y, testing=True)
+        outputs = self.call_train_test(x, y, sample_weight=sample_weight, testing=True)
 
         if getattr(self, "pre_eval_topk", None) is not None:
             # During eval, the retrieval-task only returns positive scores
             # so we need to retrieve top-k negative scores to compute the loss
             outputs = self.pre_eval_topk.call_outputs(outputs)
 
-        self.compute_loss(x, outputs.targets, outputs.predictions, sample_weight)
-        metrics = self.compute_metrics(outputs, sample_weight, training=False)
+        self.compute_loss(x, outputs.targets, outputs.predictions, outputs.sample_weight)
+        metrics = self.compute_metrics(outputs, training=False)
         # Adding regularization loss to metrics
         metrics["regularization_loss"] = tf.reduce_sum(cast_losses_to_common_dtype(self.losses))
 
@@ -428,7 +617,6 @@ class BaseModel(tf.keras.Model):
     def compute_metrics(
         self,
         prediction_outputs: PredictionOutput,
-        sample_weight: tf.Tensor,
         training: bool,
     ) -> Dict[str, tf.Tensor]:
         """Overrides Model.compute_metrics() for some custom behaviour
@@ -439,8 +627,6 @@ class BaseModel(tf.keras.Model):
         ----------
         prediction_outputs : PredictionOutput
             Contains properties with targets and predictions
-        sample_weight : tf.Tensor
-            Sample weights for weighted metrics
         training : bool
             Flag that indicates if metrics are being computed during
             training or evaluation
@@ -464,11 +650,14 @@ class BaseModel(tf.keras.Model):
             # Providing label_relevant_counts for TopkMetrics, as metric.update_state()
             # should have standard signature for better compatibility with Keras methods
             # like self.compiled_metrics.update_state()
-            for topk_metric in filter_topk_metrics(self.compiled_metrics.metrics):
-                topk_metric.label_relevant_counts = prediction_outputs.label_relevant_counts
+            if hasattr(prediction_outputs, "label_relevant_counts"):
+                for topk_metric in filter_topk_metrics(self.compiled_metrics.metrics):
+                    topk_metric.label_relevant_counts = prediction_outputs.label_relevant_counts
 
             self.compiled_metrics.update_state(
-                prediction_outputs.targets, prediction_outputs.predictions, sample_weight
+                prediction_outputs.targets,
+                prediction_outputs.predictions,
+                prediction_outputs.sample_weight,
             )
         # Returns the current value of metrics
         metrics = self.metrics_results()
@@ -669,6 +858,7 @@ class Model(BaseModel):
             block.schema for block in self.submodules if getattr(block, "is_input", False)
         ]
         self.schema = sum(input_block_schemas, Schema())
+        self._frozen_blocks = set()
 
     def _maybe_build(self, inputs):
         if isinstance(inputs, dict):
@@ -719,7 +909,7 @@ class Model(BaseModel):
 
     def call(self, inputs, targets=None, training=False, testing=False, output_context=False):
         context = self._create_context(
-            AsDenseFeatures()(inputs),  # TODO: Change this to ragged
+            AsRaggedFeatures()(inputs),
             targets=targets,
             training=training,
             testing=testing,
@@ -832,7 +1022,7 @@ class Model(BaseModel):
         return cls(*layers, pre=pre, post=post)
 
     def get_config(self):
-        config = maybe_deserialize_keras_objects({}, ["pre", "post"])
+        config = maybe_serialize_keras_objects(self, {}, ["pre", "post"])
         for i, layer in enumerate(self.blocks):
             config[i] = tf.keras.utils.serialize_keras_object(layer)
 
@@ -848,6 +1038,152 @@ class Model(BaseModel):
             _arg_spec = self._saved_model_arg_spec
             self._saved_model_arg_spec = ([_arg_spec[0][0]], _arg_spec[1])
 
+    @property
+    def frozen_blocks(self):
+        """
+        Get frozen blocks of model, only on which you called freeze_blocks before, the result dose
+        not include those blocks frozen in other methods, for example, if you create the embedding
+        and set the `trainable` as False, it would not be tracked by this property, but you can also
+        call unfreeze_blocks on those blocks.
+
+        Please note that sub-block of self._frozen_blocks is also frozen, but not recorded by this
+        variable, because if you want to unfreeze the whole model, you only need to unfreeze blocks
+        you froze before (called freeze_blocks before), this function would unfreeze all sub-blocks
+        recursively and automatically.
+
+        If you want to get all frozen blocks and sub-blocks of the model:
+            `get_sub_blocks(model.frozen_blocks)`
+        """
+        return list(self._frozen_blocks)
+
+    def freeze_blocks(
+        self,
+        blocks: Union[Sequence[Block], Sequence[str]],
+    ):
+        """Freeze all sub-blocks of given blocks recursively. Please make sure to compile the model
+        after freezing.
+
+        Important note about layer-freezing: Calling `compile()` on a model is meant to "save" the
+        behavior of that model, which means that whether the layer is frozen or not would be
+        preserved for the model, so if you want to freeze any layer of the model, please make sure
+        to compile it again.
+
+        TODO: Make it work for graph mode. Now if model compile and fit for multiple times with
+        graph mode (run_eagerly=True) could raise TensorFlow error. Please find example in
+        test_freeze_parallel_block.
+
+        Parameters
+        ----------
+        blocks : Union[Sequence[Block], Sequence[str]]
+            Blocks or names of blocks to be frozen
+
+        Example :
+            ```python
+            input_block = ml.InputBlockV2(schema)
+            layer_1 = ml.MLPBlock([64], name="layer_1")
+            layer_2 = ml.MLPBlock([1], no_activation_last_layer=True, name="layer_2")
+            two_layer = ml.SequentialBlock([layer_1, layer_2], name="two_layers")
+            body = input_block.connect(two_layer)
+            model = ml.Model(body, ml.BinaryClassificationTask("click"))
+
+            # Compile(Make sure set run_eagerly mode) and fit -> model.freeze_blocks -> compile and
+            # fit Set run_eagerly=True in order to avoid error: "Called a function referencing
+            # variables which have been deleted". Model needs to be built by fit or build.
+
+            model.compile(run_eagerly=True, optimizer=tf.keras.optimizers.SGD(lr=0.1))
+            model.fit(ecommerce_data, batch_size=128, epochs=1)
+
+            model.freeze_blocks(["user_categories", "layer_2"])
+            # From the result of model.summary(), you can find which block is frozen (trainable: N)
+            print(model.summary(expand_nested=True, show_trainable=True, line_length=80))
+
+            model.compile(run_eagerly=False, optimizer="adam")
+            model.fit(ecommerce_data, batch_size=128, epochs=10)
+            ```
+
+        """
+        if not isinstance(blocks, (list, tuple)):
+            blocks = [blocks]
+        if isinstance(blocks[0], str):
+            blocks_to_freeze = self.get_blocks_by_name(blocks)
+        elif isinstance(blocks[0], Block):
+            blocks_to_freeze = blocks
+        for b in blocks_to_freeze:
+            b.trainable = False
+        self._frozen_blocks.update(blocks_to_freeze)
+
+    def unfreeze_blocks(
+        self,
+        blocks: Union[Sequence[Block], Sequence[str]],
+    ):
+        """
+        Unfreeze all sub-blocks of given blocks recursively
+
+        Important note about layer-freezing: Calling `compile()` on a model is meant to "save" the
+        behavior of that model, which means that whether the layer is frozen or not would be
+        preserved for the model, so if you want to freeze any layer of the model, please make sure
+        to compile it again.
+        """
+        if not isinstance(blocks, (list, tuple)):
+            blocks = [blocks]
+        if isinstance(blocks[0], Block):
+            blocks_to_unfreeze = set(get_sub_blocks(blocks))
+        elif isinstance(blocks[0], str):
+            blocks_to_unfreeze = self.get_blocks_by_name(blocks)
+        for b in blocks_to_unfreeze:
+            if b not in self._frozen_blocks:
+                warnings.warn(
+                    f"Block or sub-block {b} was not frozen when calling unfreeze_block("
+                    f"{blocks})."
+                )
+            else:
+                self._frozen_blocks.remove(b)
+            b.trainable = True
+
+    def unfreeze_all_frozen_blocks(self):
+        """
+        Unfreeze all blocks (including blocks and sub-blocks) of this model recursively
+
+        Important note about layer-freezing: Calling `compile()` on a model is meant to "save" the
+        behavior of that model, which means that whether the layer is frozen or not would be
+        preserved for the model, so if you want to freeze any layer of the model, please make sure
+        to compile it again.
+        """
+        for b in self._frozen_blocks:
+            b.trainable = True
+        self._frozen_blocks = set()
+
+    def get_blocks_by_name(self, block_names: Sequence[str]) -> List[Block]:
+        """Get blocks by given block_names, return a list of blocks
+        Traverse(Iterate) the model to check each block (sub_block) by BFS"""
+        result_blocks = set()
+        if not isinstance(block_names, (list, tuple)):
+            block_names = [block_names]
+
+        for block in self.blocks:
+            # Traversse all submodule (BFS) except ModelContext
+            deque = collections.deque()
+            if not isinstance(block, ModelContext):
+                deque.append(block)
+            while deque:
+                current_module = deque.popleft()
+                # Already found all blocks
+                if len(block_names) == 0:
+                    break
+                # Found a block
+                if current_module.name in block_names:
+                    result_blocks.add(current_module)
+                    block_names.remove(current_module.name)
+                for sub_module in current_module._flatten_modules(
+                    include_self=False, recursive=False
+                ):
+                    # Filter out modelcontext
+                    if type(sub_module) != ModelContext:
+                        deque.append(sub_module)
+            if len(block_names) > 0:
+                raise ValueError(f"Cannot find block with the name of {block_names}")
+        return list(result_blocks)
+
 
 @runtime_checkable
 class RetrievalBlock(Protocol):
