@@ -65,10 +65,11 @@ class PredictionTask(Layer, ContextMixin):
         name: Optional[Text] = None,
         **kwargs,
     ) -> None:
-        super().__init__(name=name, **kwargs)
         self.target_name = target_name
-        self.task_block = task_block
         self._task_name = task_name
+        name = name or self.task_name
+        super().__init__(name=name, **kwargs)
+        self.task_block = task_block
         self.pre = pre
         self._pre_eval_topk = pre_eval_topk
 
@@ -105,7 +106,7 @@ class PredictionTask(Layer, ContextMixin):
             x = self.task_block(x)
 
         if self.pre:
-            x = self.pre(inputs, **kwargs)
+            x = self.pre(x, **kwargs)
 
         return x
 
@@ -208,6 +209,13 @@ class PredictionTask(Layer, ContextMixin):
 
         return config
 
+    def create_default_metrics(self):
+        metrics = []
+        for metric in self.DEFAULT_METRICS:
+            metrics.append(metric(name=self.child_name(to_snake_case(metric.__name__))))
+
+        return metrics
+
 
 @tf.keras.utils.register_keras_serializable(package="merlin.models")
 class ParallelPredictionBlock(ParallelBlock):
@@ -230,27 +238,50 @@ class ParallelPredictionBlock(ParallelBlock):
         *prediction_tasks: PredictionTask,
         task_blocks: Optional[Union[Layer, Dict[str, Layer]]] = None,
         bias_block: Optional[Layer] = None,
+        task_weights=None,
         pre: Optional[BlockType] = None,
         post: Optional[BlockType] = None,
         **kwargs,
     ):
         self.prediction_tasks = prediction_tasks
         self.bias_block = bias_block
-        self.bias_logit = tf.keras.layers.Dense(1)
+        if bias_block:
+            self.bias_logit = tf.keras.layers.Dense(1)
 
         self.prediction_task_dict = {}
         if prediction_tasks:
             for task in prediction_tasks:
                 self.prediction_task_dict[task.task_name] = task
 
-        super(ParallelPredictionBlock, self).__init__(self.prediction_task_dict, pre=pre, post=post)
+        super(ParallelPredictionBlock, self).__init__(
+            self.prediction_task_dict, pre=pre, post=post, use_layer_name=False, **kwargs
+        )
 
         if task_blocks:
             self._set_task_blocks(task_blocks)
 
     @classmethod
-    def get_tasks_from_schema(cls, schema, task_weight_dict=None):
+    def get_tasks_from_schema(
+        cls,
+        schema,
+        task_weight_dict: Optional[Dict[str, float]] = None,
+        task_pre_dict: Optional[Dict[str, Block]] = None,
+    ):
+        """Built Multi-task prediction Block from schema
+
+        Parameters
+        ----------
+        schema : Schema
+            The `Schema` with the input features
+        task_weight_dict : Optional[Dict[str, float]], optional
+            Dict where keys are target feature names and values are weights for each task,
+            by default None
+        task_pre_dict: Optional[Dict[str, Block]], optional
+            Dict where keys are target feature names and values are Blocks to be used as
+            pre for those tasks
+        """
         task_weight_dict = task_weight_dict or {}
+        task_pre_dict = task_pre_dict or {}
 
         tasks: List[PredictionTask] = []
         task_weights = []
@@ -258,10 +289,14 @@ class ParallelPredictionBlock(ParallelBlock):
         from merlin.models.tf.prediction_tasks.regression import RegressionTask
 
         for binary_target in schema.select_by_tag(Tags.BINARY_CLASSIFICATION).column_names:
-            tasks.append(BinaryClassificationTask(binary_target))
+            tasks.append(
+                BinaryClassificationTask(binary_target, pre=task_pre_dict.get(binary_target, None))
+            )
             task_weights.append(task_weight_dict.get(binary_target, 1.0))
         for regression_target in schema.select_by_tag(Tags.REGRESSION).column_names:
-            tasks.append(RegressionTask(regression_target))
+            tasks.append(
+                RegressionTask(regression_target, pre=task_pre_dict.get(regression_target, None))
+            )
             task_weights.append(task_weight_dict.get(regression_target, 1.0))
         # TODO: Add multi-class classification here. Figure out how to get number of classes
 
@@ -273,6 +308,7 @@ class ParallelPredictionBlock(ParallelBlock):
         schema: Schema,
         task_blocks: Optional[Union[Layer, Dict[str, Layer]]] = None,
         task_weight_dict: Optional[Dict[str, float]] = None,
+        task_pre_dict: Optional[Dict[str, Block]] = None,
         bias_block: Optional[Layer] = None,
         **kwargs,
     ) -> "ParallelPredictionBlock":
@@ -285,13 +321,17 @@ class ParallelPredictionBlock(ParallelBlock):
         task_blocks : Optional[Union[Layer, Dict[str, Layer]]], optional
             Task blocks to be used for prediction, by default None
         task_weight_dict : Optional[Dict[str, float]], optional
-            Weights for each task, by default None
+            Dict where keys are target feature names and values are weights for each task,
+            by default None
+        task_pre_dict: Optional[Dict[str, Block]], optional
+            Dict where keys are target feature names and values are Blocks to be used as pre
+            for those tasks
         bias_block : Optional[Layer], optional
             Bias block to be used for prediction, by default None
         """
         task_weight_dict = task_weight_dict or {}
 
-        task_weights, tasks = cls.get_tasks_from_schema(schema, task_weight_dict)
+        task_weights, tasks = cls.get_tasks_from_schema(schema, task_weight_dict, task_pre_dict)
 
         return cls(
             *tasks,
@@ -418,6 +458,7 @@ class ParallelPredictionBlock(ParallelBlock):
             config["schema"] = tensorflow_metadata_json_to_schema(config["schema"])
 
         prediction_tasks = config.pop("prediction_tasks", [])
+        config.pop("parallel_layers", None)
 
         return cls(*prediction_tasks, **config)
 
